use anyhow::{anyhow, Result};
use serde::Deserialize;

#[derive(Deserialize, Debug, PartialEq, Eq, Clone)]
pub struct UpResp {
    pub files: Vec<String>,
}

#[derive(Deserialize, Debug, PartialEq, Eq, Clone)]
pub struct CatsResp {
    pub categories: Vec<Category>,
}

#[derive(Deserialize, Debug, PartialEq, Eq, Clone)]
pub struct UsersResp {
    pub users: Vec<User>,
}

#[derive(Deserialize, Debug, PartialEq, Eq, Clone)]
pub struct MemesResp {
    pub memes: Vec<Meme>,
}

#[derive(Deserialize, Debug, PartialEq, Eq, Clone)]
pub struct Category {
    pub id: String,
    pub name: String,
}

#[derive(Deserialize, Debug, PartialEq, Eq, Clone)]
pub struct Meme {
    pub id: String,
    pub link: String,
    pub path: String,
    pub category: String,
    pub user: String,
}

impl Meme {
    pub fn file_name(&self) -> Result<&str> {
        self.path
            .split('/')
            .last()
            .ok_or_else(|| anyhow!("failed to get file name. server response invalid"))
    }
}

#[derive(Deserialize, Debug, PartialEq, Eq, Clone)]
pub struct User {
    pub name: String,
    pub id: Option<String>,
    pub tokenhash: Option<String>,
    pub userdir: Option<String>,
    pub dayuploads: String,
}

impl User {
    pub fn get_id(&self) -> Option<&String> {
        self.id
            .as_ref()
            .or(self.tokenhash.as_ref())
            .or(self.userdir.as_ref())
    }
}
